# 5x3 digits A-Z 64 Neopixel display
# Tony Goodhew 23rd March 2022
import array, time
from machine import Pin
import rp2
import random
# Configure the number of WS2812 LEDs.
NUM_LEDS = 64
PIN_NUM = 16
brightness = 0.1

@rp2.asm_pio(sideset_init=rp2.PIO.OUT_LOW, out_shiftdir=rp2.PIO.SHIFT_LEFT, autopull=True, pull_thresh=24)
def ws2812():
    T1 = 2
    T2 = 5
    T3 = 3
    wrap_target()
    label("bitloop")
    out(x, 1)               .side(0)    [T3 - 1]
    jmp(not_x, "do_zero")   .side(1)    [T1 - 1]
    jmp("bitloop")          .side(1)    [T2 - 1]
    label("do_zero")
    nop()                   .side(0)    [T2 - 1]
    wrap()


# Create the StateMachine with the ws2812 program, outputting on pin
sm = rp2.StateMachine(0, ws2812, freq=8_000_000, sideset_base=Pin(PIN_NUM))

# Start the StateMachine, it will wait for data on its FIFO.
sm.active(1)

# Display a pattern on the LEDs via an array of LED RGB values.
ar = array.array("I", [0 for _ in range(NUM_LEDS)])

##########################################################################
def pixels_show():
    dimmer_ar = array.array("I", [0 for _ in range(NUM_LEDS)])
    for i,c in enumerate(ar):
        r = int(((c >> 8) & 0xFF) * brightness)
        g = int(((c >> 16) & 0xFF) * brightness)
        b = int((c & 0xFF) * brightness)
        dimmer_ar[i] = (g<<16) + (r<<8) + b
    sm.put(dimmer_ar, 8)
    time.sleep_ms(10)

def pixels_set(i, color):
    ar[i] = (color[1]<<16) + (color[0]<<8) + color[2]
    
def xy_set(x, y, color):
    pos = x + y*8
    pixels_set(pos, color)

def pixels_fill(color):
    for i in range(len(ar)):
        pixels_set(i, color)

def clear():
    color = (0,0,0)
    pixels_fill(color)

def flag():
    for y in range(4):
        for x in range(8):
            xy_set(x,y,(0,0,255))
            xy_set(x,y+4,(255,255,0))
    pixels_show()
    
#Global variables

#one row per 5x3 digit - 0 to 9 and T
nums =[1,1,1,1,0,1,1,0,1,1,0,1,1,1,1,  # 0
       0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,  # 1
       1,1,1,0,0,1,0,1,0,1,0,0,1,1,1,  # 2
       1,1,1,0,0,1,1,1,1,0,0,1,1,1,1,  # 3
       1,0,0,1,0,1,1,1,1,0,0,1,0,0,1,  # 4
       1,1,1,1,0,0,1,1,1,0,0,1,1,1,1,  # 5
       1,1,1,1,0,0,1,1,1,1,0,1,1,1,1,  # 6
       1,1,1,0,0,1,0,1,0,1,0,0,1,0,0,  # 7
       1,1,1,1,0,1,1,1,1,1,0,1,1,1,1,  # 8
       1,1,1,1,0,1,1,1,1,0,0,1,0,0,1,  # 9
       0,1,0,1,0,1,1,1,1,1,0,1,1,0,1,  # A
       1,1,0,1,0,1,1,1,0,1,0,1,1,1,0,  # B
       0,1,1,1,0,0,1,0,0,1,0,0,0,1,1,  # C
       1,1,0,1,0,1,1,0,1,1,0,1,1,1,0,  # D
       1,1,1,1,0,0,1,1,1,1,0,0,1,1,1,  # E
       1,1,1,1,0,0,1,1,1,1,0,0,1,0,0]  # F

powers = [1,2,4,8,16,32,64,128] # For Bin and Hex

def show_num(val,xd,yd,r,g,b):     # Single digit
    offset = val * 15
    for p in range(offset,offset + 15):
        if nums[p] == 1:
            xt = p % 3
            yt = (p-offset) // 3
            xy_set(xt+xd,yt+yd,(r,g,b))       
    
def show_number(val,r,g,b):  #Base 10 - tens and units
    abs_val = abs(val)
    tens = abs_val // 10
    units = abs_val % 10
    clear()
    if (abs_val > 9): show_num(tens,0,2,r,g,b)
    show_num(units,4,2,r,g,b)
    if abs_val == 100: # Deal with 3 digit "100" - special case
        clear()
        show_num(0,5,2,r,g,b)
        show_num(0,2,2,r,g,b)
        for i in range(2,7):
            xy_set(0,i,(r,g,b))
    if val < 0 :       # Display 'Negative' bar
        for i in range(0,8):
            xy_set(i,0,(0,0,128))

def show_time(r,g,b):   
    for i in range(0,10):
        clear()
        tens = hrs // 10
        units = hrs % 10
        show_num(tens,0,2,r,g,b)
        show_num(units,4,2,r,g,b)
        show_colon(7,3)
        pixels_show()
        time.sleep(0.9)
        clear()
        tens = mins // 10
        units = mins % 10
        show_num(tens,1,2,r,g,b)
        show_num(units,5,2,r,g,b)
        show_colon(0,3)
        pixels_show()
        time.sleep(0.9)

def show_colon(x,y):
    xy_set(x,y,(128,0,0))
    xy_set(x,y+2,(128,0,0))

def show_block(x,y,r,g,b):
    xy_set(x,y,(0,0,128))
    xy_set(x+1,y,(0,0,128))

def show_large(val,r,g,b): #Range 0 to 399
    orig_val = val
    hundreds = val // 100
    val = val - hundreds * 100
    tens = val // 10
    units = val % 10
    clear()
    if (orig_val > 9): show_num(tens,0,2,r,g,b)
    show_num(units,4,2,r,g,b)
    # Blobs on top line to indicate hundreds
    if hundreds >0: show_block(3,0,0,0,128) #Centre - 100
    if hundreds >1: show_block(0,0,0,0,128) #Right  - 200
    if hundreds >2: show_block(6,0,0,0,128) #Left   - 300

def show_binary(val,r,g,b):
    for index in range(0, 8):
        if powers[index] & val:
            xy_set(7-index,3,(r,g,b))
            pixels_show()

def show_hex(val,r,g,b):
      nibble_hi = val // 16
      nibble_low = val % 16
      clear()
      show_num(nibble_hi,1,2,r,g,b)
      show_num(nibble_low,5,2,r,g,b)
      pixels_show()

#========== MAIN ===============
flag()
time.sleep(2)
clear()
pixels_show()

print("** Small digits for A-Z 64 Neopixel Square **")
print("**      Tony Goodhew -- 23 March 2022      **\n")

#Counting from -100 to +100
print("Counting from -100 to 100")
print("Can be used for percentages,")
print("such as Humidity, or temperature")
for i in range(-100,101):
    show_number(i,128,50,30)
    pixels_show()
    time.sleep(0.1)
    if abs(i) == 100: time.sleep(1.3) # Longer for "100"
clear()

#Display current time HH:MM - alternating HH: & :MM
print("\nDisplay time")
hrs = 14
mins = 51
show_time(0,128,30)
clear()
print("")

#15 Random numbers between 0 and 399
print("15 numbers in range 0 to 399 inclusive")
print("Can be used for angles 0 to 360 degrees\n")
for i in range(0,13):
    val = random.randint(0,399)
    print(val)
    show_large(val,0,0,128)
    pixels_show()
    time.sleep(1.7)
    clear()

print(7)
show_large(7,0,0,128) # Suppressed leading zero
pixels_show()
time.sleep(1.7)
clear()
print(107)
show_large(107,0,0,128) # Essential leading zero
pixels_show()
time.sleep(1.7)

print("\nBinary - base 2")
clear()
for val in range(0,256):
    clear()
    show_binary(val,200,200,0)
    time.sleep(0.07)
time.sleep(0.15)
clear()

print("\nHexadecimal - base 16")
clear()
for val in range(0,256):
    clear()
    show_hex(val,100,100,0)
    time.sleep(.08)
time.sleep(0.9)
clear()
pixels_show()

flag()
time.sleep(5)
clear()
pixels_show()
print("All done ##############")
print("\nDone")
